# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'scanToolTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(1133, 883)
        self.gridLayout_3 = QtWidgets.QGridLayout(Form)
        self.gridLayout_3.setContentsMargins(6, 6, 6, 6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.splitter = QtWidgets.QSplitter(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setStyleSheet("QSplitter::handle {image: url(:/icons/icons/splitter.png);}")
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setOpaqueResize(True)
        self.splitter.setHandleWidth(12)
        self.splitter.setChildrenCollapsible(True)
        self.splitter.setObjectName("splitter")
        self.widget_scan_settings = QtWidgets.QWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_scan_settings.sizePolicy().hasHeightForWidth())
        self.widget_scan_settings.setSizePolicy(sizePolicy)
        self.widget_scan_settings.setObjectName("widget_scan_settings")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_scan_settings)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label = QtWidgets.QLabel(self.widget_scan_settings)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.combobox_template = QtWidgets.QComboBox(self.widget_scan_settings)
        self.combobox_template.setObjectName("combobox_template")
        self.combobox_template.addItem("")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.combobox_template)
        self.gridLayout.addLayout(self.formLayout_2, 1, 0, 1, 1)
        self.label_scan_settings = QtWidgets.QLabel(self.widget_scan_settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_scan_settings.sizePolicy().hasHeightForWidth())
        self.label_scan_settings.setSizePolicy(sizePolicy)
        self.label_scan_settings.setStyleSheet("font-weight:bold;background:#f6b442;padding:5")
        self.label_scan_settings.setObjectName("label_scan_settings")
        self.gridLayout.addWidget(self.label_scan_settings, 0, 0, 1, 1)
        self.stackedWidget = QtWidgets.QStackedWidget(self.widget_scan_settings)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_scan_settings = QtWidgets.QWidget()
        self.page_scan_settings.setObjectName("page_scan_settings")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.page_scan_settings)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_start_scan = QtWidgets.QPushButton(self.page_scan_settings)
        self.button_start_scan.setStatusTip("")
        self.button_start_scan.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/play.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_start_scan.setIcon(icon)
        self.button_start_scan.setIconSize(QtCore.QSize(16, 16))
        self.button_start_scan.setObjectName("button_start_scan")
        self.horizontalLayout.addWidget(self.button_start_scan)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.button_remove = QtWidgets.QPushButton(self.page_scan_settings)
        self.button_remove.setStyleSheet("QPushButton { background-color: #ff4e00 }\\\\nQPushButton:pressed { background-color: #ff4e00 }\\\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\\nQPushButton:disabled{ background:silver;color:grey }")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_remove.setIcon(icon1)
        self.button_remove.setIconSize(QtCore.QSize(16, 16))
        self.button_remove.setObjectName("button_remove")
        self.horizontalLayout.addWidget(self.button_remove)
        self.gridLayout_4.addLayout(self.horizontalLayout, 3, 0, 1, 1)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtWidgets.QLabel(self.page_scan_settings)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.input_comment = QtWidgets.QLineEdit(self.page_scan_settings)
        self.input_comment.setObjectName("input_comment")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.input_comment)
        self.label_3 = QtWidgets.QLabel(self.page_scan_settings)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.input_output = QtWidgets.QLineEdit(self.page_scan_settings)
        self.input_output.setReadOnly(True)
        self.input_output.setObjectName("input_output")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.input_output)
        self.label_4 = QtWidgets.QLabel(self.page_scan_settings)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.input_repeat_scan = QtWidgets.QDoubleSpinBox(self.page_scan_settings)
        self.input_repeat_scan.setDecimals(0)
        self.input_repeat_scan.setMaximum(2140000000.0)
        self.input_repeat_scan.setObjectName("input_repeat_scan")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.input_repeat_scan)
        self.gridLayout_4.addLayout(self.formLayout, 1, 0, 1, 1)
        self.scroll_area_scan = QtWidgets.QScrollArea(self.page_scan_settings)
        self.scroll_area_scan.setWidgetResizable(True)
        self.scroll_area_scan.setObjectName("scroll_area_scan")
        self.widget_scan = QtWidgets.QWidget()
        self.widget_scan.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.widget_scan.setObjectName("widget_scan")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget_scan)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scroll_area_scan.setWidget(self.widget_scan)
        self.gridLayout_4.addWidget(self.scroll_area_scan, 2, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_scan_settings)
        self.page_template_designer = QtWidgets.QWidget()
        self.page_template_designer.setObjectName("page_template_designer")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.page_template_designer)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.scroll_area_custom_template = QtWidgets.QScrollArea(self.page_template_designer)
        self.scroll_area_custom_template.setWidgetResizable(True)
        self.scroll_area_custom_template.setObjectName("scroll_area_custom_template")
        self.widget_custom_template = QtWidgets.QWidget()
        self.widget_custom_template.setGeometry(QtCore.QRect(0, 0, 273, 500))
        self.widget_custom_template.setObjectName("widget_custom_template")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.widget_custom_template)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.scroll_area_custom_template.setWidget(self.widget_custom_template)
        self.gridLayout_5.addWidget(self.scroll_area_custom_template, 0, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.button_reset_template = QtWidgets.QPushButton(self.page_template_designer)
        self.button_reset_template.setStatusTip("")
        self.button_reset_template.setStyleSheet("QPushButton { background-color: #ff4e00 }\\\\nQPushButton:pressed { background-color: #ff4e00 }\\\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\\nQPushButton:disabled{ background:silver;color:grey }")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/reset_bw.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_reset_template.setIcon(icon2)
        self.button_reset_template.setIconSize(QtCore.QSize(16, 16))
        self.button_reset_template.setObjectName("button_reset_template")
        self.horizontalLayout_5.addWidget(self.button_reset_template)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.button_save_template = QtWidgets.QPushButton(self.page_template_designer)
        self.button_save_template.setStyleSheet("QPushButton { background-color: #00b4ff }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/save_bw.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_save_template.setIcon(icon3)
        self.button_save_template.setIconSize(QtCore.QSize(16, 16))
        self.button_save_template.setObjectName("button_save_template")
        self.horizontalLayout_5.addWidget(self.button_save_template)
        self.button_import_template = QtWidgets.QPushButton(self.page_template_designer)
        self.button_import_template.setStyleSheet("QPushButton { background-color: #00b4ff }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/icons/load_bw.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_import_template.setIcon(icon4)
        self.button_import_template.setIconSize(QtCore.QSize(16, 16))
        self.button_import_template.setObjectName("button_import_template")
        self.horizontalLayout_5.addWidget(self.button_import_template)
        self.gridLayout_5.addLayout(self.horizontalLayout_5, 5, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.button_add_scan_step = QtWidgets.QPushButton(self.page_template_designer)
        self.button_add_scan_step.setStatusTip("")
        self.button_add_scan_step.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        self.button_add_scan_step.setIconSize(QtCore.QSize(16, 16))
        self.button_add_scan_step.setObjectName("button_add_scan_step")
        self.horizontalLayout_4.addWidget(self.button_add_scan_step)
        self.button_add_step = QtWidgets.QPushButton(self.page_template_designer)
        self.button_add_step.setStatusTip("")
        self.button_add_step.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        self.button_add_step.setIconSize(QtCore.QSize(16, 16))
        self.button_add_step.setObjectName("button_add_step")
        self.horizontalLayout_4.addWidget(self.button_add_step)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.gridLayout_5.addLayout(self.horizontalLayout_4, 2, 0, 1, 1)
        self.label_output_log = QtWidgets.QLabel(self.page_template_designer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_output_log.sizePolicy().hasHeightForWidth())
        self.label_output_log.setSizePolicy(sizePolicy)
        self.label_output_log.setStyleSheet("font-weight:bold;background:#f6b442;padding:5")
        self.label_output_log.setObjectName("label_output_log")
        self.gridLayout_5.addWidget(self.label_output_log, 3, 0, 1, 1)
        self.list_measurement_groups = QtWidgets.QListWidget(self.page_template_designer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_measurement_groups.sizePolicy().hasHeightForWidth())
        self.list_measurement_groups.setSizePolicy(sizePolicy)
        self.list_measurement_groups.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.list_measurement_groups.setObjectName("list_measurement_groups")
        self.gridLayout_5.addWidget(self.list_measurement_groups, 4, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_template_designer)
        self.gridLayout.addWidget(self.stackedWidget, 3, 0, 1, 1)
        self.widget_scan_output = QtWidgets.QWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_scan_output.sizePolicy().hasHeightForWidth())
        self.widget_scan_output.setSizePolicy(sizePolicy)
        self.widget_scan_output.setObjectName("widget_scan_output")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget_scan_output)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.widget = QtWidgets.QWidget(self.widget_scan_output)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_status = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_status.sizePolicy().hasHeightForWidth())
        self.label_status.setSizePolicy(sizePolicy)
        self.label_status.setMinimumSize(QtCore.QSize(100, 0))
        self.label_status.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        font.setKerning(True)
        self.label_status.setFont(font)
        self.label_status.setStyleSheet("QLabel {background:#68d532}")
        self.label_status.setAlignment(QtCore.Qt.AlignCenter)
        self.label_status.setObjectName("label_status")
        self.horizontalLayout_2.addWidget(self.label_status)
        self.progress_bar_scan = QtWidgets.QProgressBar(self.widget)
        self.progress_bar_scan.setProperty("value", 0)
        self.progress_bar_scan.setObjectName("progress_bar_scan")
        self.horizontalLayout_2.addWidget(self.progress_bar_scan)
        self.label_scan_step = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_scan_step.sizePolicy().hasHeightForWidth())
        self.label_scan_step.setSizePolicy(sizePolicy)
        self.label_scan_step.setMinimumSize(QtCore.QSize(70, 0))
        self.label_scan_step.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label_scan_step.setStyleSheet("QLabel {background:#fff}")
        self.label_scan_step.setAlignment(QtCore.Qt.AlignCenter)
        self.label_scan_step.setObjectName("label_scan_step")
        self.horizontalLayout_2.addWidget(self.label_scan_step)
        self.label_running_time = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_running_time.sizePolicy().hasHeightForWidth())
        self.label_running_time.setSizePolicy(sizePolicy)
        self.label_running_time.setMinimumSize(QtCore.QSize(70, 0))
        self.label_running_time.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label_running_time.setStyleSheet("QLabel {background:#fff}")
        self.label_running_time.setAlignment(QtCore.Qt.AlignCenter)
        self.label_running_time.setObjectName("label_running_time")
        self.horizontalLayout_2.addWidget(self.label_running_time)
        self.label_remaining_time = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_remaining_time.sizePolicy().hasHeightForWidth())
        self.label_remaining_time.setSizePolicy(sizePolicy)
        self.label_remaining_time.setMinimumSize(QtCore.QSize(70, 0))
        self.label_remaining_time.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label_remaining_time.setStyleSheet("QLabel {background:#fff}")
        self.label_remaining_time.setAlignment(QtCore.Qt.AlignCenter)
        self.label_remaining_time.setObjectName("label_remaining_time")
        self.horizontalLayout_2.addWidget(self.label_remaining_time)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.label_estimated_finish = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_estimated_finish.sizePolicy().hasHeightForWidth())
        self.label_estimated_finish.setSizePolicy(sizePolicy)
        self.label_estimated_finish.setMinimumSize(QtCore.QSize(70, 0))
        self.label_estimated_finish.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label_estimated_finish.setStyleSheet("QLabel {background:#fff}")
        self.label_estimated_finish.setAlignment(QtCore.Qt.AlignCenter)
        self.label_estimated_finish.setObjectName("label_estimated_finish")
        self.horizontalLayout_2.addWidget(self.label_estimated_finish)
        self.button_stop_scan = QtWidgets.QPushButton(self.widget)
        self.button_stop_scan.setStyleSheet("QPushButton { background-color: #ff4e00 }\\\\nQPushButton:pressed { background-color: #ff4e00 }\\\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\\nQPushButton:disabled{ background:silver;color:grey }")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/icons/stop.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_stop_scan.setIcon(icon5)
        self.button_stop_scan.setIconSize(QtCore.QSize(16, 16))
        self.button_stop_scan.setObjectName("button_stop_scan")
        self.horizontalLayout_2.addWidget(self.button_stop_scan)
        self.button_resume_scan = QtWidgets.QPushButton(self.widget)
        self.button_resume_scan.setStyleSheet("QPushButton { background-color: #68d532 }\n"
"QPushButton:pressed { background-color: #28a900 }\n"
"QPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/icons/resume.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_resume_scan.setIcon(icon6)
        self.button_resume_scan.setIconSize(QtCore.QSize(16, 16))
        self.button_resume_scan.setObjectName("button_resume_scan")
        self.horizontalLayout_2.addWidget(self.button_resume_scan)
        self.button_pause_scan = QtWidgets.QPushButton(self.widget)
        self.button_pause_scan.setStyleSheet("QPushButton { background-color: #00b4ff }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/icons/pause.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_pause_scan.setIcon(icon7)
        self.button_pause_scan.setIconSize(QtCore.QSize(16, 16))
        self.button_pause_scan.setObjectName("button_pause_scan")
        self.horizontalLayout_2.addWidget(self.button_pause_scan)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout_2.addWidget(self.widget, 4, 0, 1, 1)
        self.label_current_Scan = QtWidgets.QLabel(self.widget_scan_output)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_current_Scan.sizePolicy().hasHeightForWidth())
        self.label_current_Scan.setSizePolicy(sizePolicy)
        self.label_current_Scan.setStyleSheet("font-weight:bold;background:#f6b442;padding:5")
        self.label_current_Scan.setObjectName("label_current_Scan")
        self.gridLayout_2.addWidget(self.label_current_Scan, 0, 0, 1, 1)
        self.scroll_area_plot = QtWidgets.QScrollArea(self.widget_scan_output)
        self.scroll_area_plot.setWidgetResizable(True)
        self.scroll_area_plot.setObjectName("scroll_area_plot")
        self.plot_widget = QtWidgets.QWidget()
        self.plot_widget.setGeometry(QtCore.QRect(0, 0, 832, 801))
        self.plot_widget.setObjectName("plot_widget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.plot_widget)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.scroll_area_plot.setWidget(self.plot_widget)
        self.gridLayout_2.addWidget(self.scroll_area_plot, 1, 0, 1, 1)
        self.gridLayout_3.addWidget(self.splitter, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.stackedWidget.setCurrentIndex(1)
        self.button_start_scan.clicked.connect(Form.start_scan)
        self.button_pause_scan.clicked.connect(Form.pause_current_scan)
        self.button_resume_scan.clicked.connect(Form.resume_current_scan)
        self.button_add_scan_step.clicked.connect(Form.add_iterative_step)
        self.button_add_step.clicked.connect(Form.add_action_step)
        self.button_reset_template.clicked.connect(Form.reset_template_input)
        self.button_save_template.clicked.connect(Form.export_template)
        self.button_import_template.clicked.connect(Form.import_template)
        self.combobox_template.currentTextChanged['QString'].connect(Form.template_selected)
        self.button_stop_scan.clicked.connect(Form.stop_current_scan)
        self.button_remove.clicked.connect(Form.remove_template)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Scanning tool"))
        self.label.setText(_translate("Form", "Template:"))
        self.combobox_template.setItemText(0, _translate("Form", "Custom"))
        self.label_scan_settings.setText(_translate("Form", "Scan settings"))
        self.button_start_scan.setToolTip(_translate("Form", "Start scan"))
        self.button_start_scan.setText(_translate("Form", "Start"))
        self.button_remove.setToolTip(_translate("Form", "Remove selected template"))
        self.button_remove.setText(_translate("Form", "Remove"))
        self.label_2.setText(_translate("Form", "Comment:"))
        self.label_3.setText(_translate("Form", "Output file:"))
        self.label_4.setText(_translate("Form", "Repeat:"))
        self.button_reset_template.setToolTip(_translate("Form", "Reset template"))
        self.button_reset_template.setText(_translate("Form", "Reset"))
        self.button_save_template.setToolTip(_translate("Form", "Save template"))
        self.button_save_template.setText(_translate("Form", "Save"))
        self.button_import_template.setToolTip(_translate("Form", "Import template from file"))
        self.button_import_template.setText(_translate("Form", "Import"))
        self.button_add_scan_step.setToolTip(_translate("Form", "Add iterative step"))
        self.button_add_scan_step.setText(_translate("Form", "+ Iterative step"))
        self.button_add_step.setToolTip(_translate("Form", "Add action step"))
        self.button_add_step.setText(_translate("Form", "+ Action step"))
        self.label_output_log.setText(_translate("Form", "Select measurement group(s)"))
        self.list_measurement_groups.setSortingEnabled(True)
        self.label_status.setText(_translate("Form", "Idle"))
        self.progress_bar_scan.setToolTip(_translate("Form", "Current scan progress"))
        self.label_scan_step.setToolTip(_translate("Form", "Current scan step"))
        self.label_scan_step.setText(_translate("Form", "0 / 0"))
        self.label_running_time.setToolTip(_translate("Form", "Current scan running time"))
        self.label_running_time.setText(_translate("Form", "00:00:00"))
        self.label_remaining_time.setToolTip(_translate("Form", "Current scan remaining time"))
        self.label_remaining_time.setText(_translate("Form", "00:00:00"))
        self.label_estimated_finish.setToolTip(_translate("Form", "Current scan estimated finish"))
        self.label_estimated_finish.setText(_translate("Form", "00:00:00"))
        self.button_stop_scan.setToolTip(_translate("Form", "Stop current scan"))
        self.button_stop_scan.setText(_translate("Form", "Stop"))
        self.button_resume_scan.setToolTip(_translate("Form", "Resume current scan"))
        self.button_resume_scan.setText(_translate("Form", "Resume"))
        self.button_pause_scan.setToolTip(_translate("Form", "Pause current scan"))
        self.button_pause_scan.setText(_translate("Form", "Pause"))
        self.label_current_Scan.setText(_translate("Form", "Current scan"))
from kamzik3 import resource_kamzik3_rc



from PyQt5.QtCore import QAbstractListModel, Qt, QItemSelectionModel, QItemSelection


class LogViewHeaderModel(QAbstractListModel):

    def __init__(self, model_data):
        QAbstractListModel.__init__(self)
        self.model_data = model_data

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.model_data)

    def data(self, model_index, role=None):
        if role == Qt.DisplayRole:
            return self.model_data[model_index.row()]

    def flags(self, model_index):
        return Qt.ItemIsSelectable | Qt.ItemIsEnabled


class LogHeaderSelectionModel(QItemSelectionModel):

    def __init__(self, model, max_selected_rows):
        QItemSelectionModel.__init__(self, model)
        self.max_selected_rows = max_selected_rows

    def select(self, selection_item, selection_flags):
        if not isinstance(selection_item, QItemSelection) or len(selection_item.indexes()) > 1:
            return
        if selection_flags & QItemSelectionModel.Select:
            selected = len(self.selectedRows(0))
            if selection_item.indexes()[0] not in self.selectedRows(0):
                selected += 1
            if selected > self.max_selected_rows:
                first_row = self.selectedRows(0)[0]
                self.select(QItemSelection(first_row, first_row), QItemSelectionModel.Deselect)

        QItemSelectionModel.select(self, selection_item, selection_flags)



from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QLabel, QPushButton

from kamzik3.constants import *
from kamzik3.gui.attributeDeviceDisplayWidget import AttributeDeviceDisplayWidget
from kamzik3.gui.deviceWidget import DeviceWidget
from kamzik3.gui.general.stackableWidget import StackableWidget
from kamzik3.gui.templates.deviceRheodyneTemplate import Ui_Form


class DeviceRheodyneWidget(Ui_Form, DeviceWidget, StackableWidget):
    configured = False

    def __init__(self, device, config=None, parent=None):
        self.value_controls = {}
        DeviceWidget.__init__(self, device, config, parent)
        self.set_status_label(self.device.get_value(ATTR_STATUS))
        self.label_device_name.setText(self.device_id)
        self.setEnabled(False)

    @pyqtSlot()
    def slot_handle_configuration(self):
        DeviceWidget.slot_handle_configuration(self)

        active_switches = self.device.get_value(ATTR_ACTIVE_SWITCHES)
        for switch in range(active_switches):
            self.add_switch(switch + 1)

        self.add_macros()

    @pyqtSlot("PyQt_PyObject")
    def slot_set_status(self, value):
        DeviceWidget.slot_set_status(self, value)
        if value in READY_DEVICE_STATUSES:
            self.setEnabled(True)
        else:
            self.setEnabled(False)

        self.set_status_label(value)

    @pyqtSlot()
    def set_status_label(self, status):
        self.label_status.setText(status)
        self.label_status.setStyleSheet("QLabel {{background:{}}}".format(self.get_status_color(status)))

    def add_macros(self):
        row = 0
        for macro_name, _ in self.device.exposed_methods:
            if macro_name == "stop_all_tasks":
                continue

            macro_button = QPushButton(macro_name)
            macro_button.clicked.connect(lambda *args, macro_name=macro_name: getattr(self.device, macro_name)())
            self.layout_macro_grid.addWidget(macro_button, int(row / 3), row % 3)
            row += 1

    def add_switch(self, switch_number):
        switch_name = "Switch{}".format(switch_number)
        current_sample_widget = AttributeDeviceDisplayWidget(self.device, [switch_name, ATTR_CURRENT_SAMPLE])
        switch_label = QLabel(switch_name)
        switch_label.setStyleSheet("QLabel {font-weight:bold;background:#f6b442;padding:5}")
        self.layout_switch_list.addWidget(switch_label, 0, switch_number)
        self.layout_switch_list.addWidget(current_sample_widget, 1, switch_number)



import logging
from threading import Thread

from kamzik3.devices.deviceWorker import DeviceWorker
from kamzik3.snippets.snippetsYaml import YamlSerializable
from kamzik3.snippets.snippetsZmq import BalancedServer


class DeviceServer(BalancedServer, Thread, YamlSerializable):

    def __init__(self, host, port, workers_count=5, worker_class=DeviceWorker):
        self.logger = logging.getLogger(u"Console.DeviceServer@{}:{}".format(host, port))
        Thread.__init__(self)
        BalancedServer.__init__(self, host, port, workers_count, worker_class)



import json
import os
import time

import pandas as pd

from kamzik3 import DeviceError, SEP
from kamzik3.constants import *
from kamzik3.devices.deviceFileSync import DeviceFileSync
from kamzik3.snippets.snippetsDecorators import expose_method

UPDATE_INSERT_GROUPS = "Insert groups"
UPDATE_INSERT_ROW = "Insert row"
UPDATE_REMOVE_GROUP = "Remove group"
UPDATE_VALUE = "Update value"
UPDATE_REMOVE_ROW = "Remove row"

class DeviceSavedAttributes(DeviceFileSync):

    def _init_attributes(self):
        DeviceFileSync._init_attributes(self)
        self.create_attribute(ATTR_GROUPS, type=TYPE_ARRAY, readonly=True)
        attribute = self.create_attribute(ATTR_LAST_UPDATE, readonly=True)
        # We set min_broadcast_timeout to zero to make sure all messages are broadcast
        attribute.min_broadcast_timeout = 0

    def handle_configuration(self):
        start_at = time.time()
        self._config_commands()
        self._config_attributes()
        self.set_value(ATTR_FILEPATH, self.filepath)
        self.set_status(STATUS_CONFIGURED)
        self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))

    def set_filepath(self, filepath):
        """
        Set filepath for h5 source file.
        @type filepath: string
        """
        if not os.path.exists(filepath):
            try:
                hdf = pd.DataFrame(data={"Comment"})
                hdf.to_hdf(filepath, "default", format="table", complevel=1)
            except PermissionError as e:
                raise DeviceError(e)
        else:
            with pd.HDFStore(filepath) as hdf:
                self.set_value(ATTR_GROUPS, hdf.keys())

        with open(self.filepath, "rb") as fp:
            self.set_raw_value(ATTR_CONTENT, fp.read())

    @expose_method({"groups_data": "Dict with group definitions"})
    def insert_groups(self, groups_data):
        """
        Insert multiple groups into saved attributes file.
        Each group is defined by entry in dict.
        Key is path to group and value is header of table.
        Example of group:
        {"/Default/Sample": {"Color", "Comment", "X [Position][mm]", "Y [Position][mm]", "Yaw [Position][deg]"}}
        @param groups_data: dict {group_path_1: {Color, Time, Comment, Device [Attribute][Unit]}, group_path_n: {...}}
        @type groups_data: dict
        """
        for group, data in groups_data.items():
            df = pd.DataFrame(data, columns=list(data.keys()))
            df.to_hdf(self.filepath, key=group, format="table", complevel=1)
            self.set_value(ATTR_LAST_UPDATE, SEP.join((UPDATE_INSERT_GROUPS, json.dumps(groups_data))))

        self.store_dataframe()
        with pd.HDFStore(self.filepath) as hdf:
            self.set_value(ATTR_GROUPS, hdf.keys())

    @expose_method({"groups_data": "Dict with group definitions"})
    def remove_group(self, group):
        with pd.HDFStore(self.filepath) as hdf:
            for group_path in self.get_value(ATTR_GROUPS):
                if group_path.startswith(f"/{group}/"):
                    hdf.remove(group_path)
            self.set_value(ATTR_LAST_UPDATE, SEP.join((UPDATE_REMOVE_GROUP, group)))
            self.set_value(ATTR_GROUPS, hdf.keys())

        self.store_dataframe()

    @expose_method({"group": "Path to existing group", "data": "Row dict with attributes"})
    def insert_row(self, group, data, position=0):
        """
        Insert new row into group specified by group path
        @param group: group path
        @type group: string
        @param data: {row color, user comment, Value of attribute, ...}
        @type data: dict
        """
        df = pd.read_hdf(self.filepath, key=group)
        line = pd.DataFrame(data, index=[position])
        df2 = pd.concat([df.iloc[:position-1], line, df.iloc[position-1:]])
        df2.reset_index(drop=True, inplace=True)
        df2.to_hdf(self.filepath, group, format="table", complevel=1)
        self.store_dataframe()
        self.set_value(ATTR_LAST_UPDATE, SEP.join((UPDATE_INSERT_ROW, group, json.dumps(data), str(position))))

    @expose_method({"group": "Path to existing group", "row": "Row index", "column": "Column index",
                    "data": "Row dict with attributes"})
    def update_value(self, group, row, column, value):
        df = pd.read_hdf(self.filepath, key=group)
        df.loc[row, df.columns[column]] = value
        df.to_hdf(self.filepath, group, format="table", complevel=1)
        self.store_dataframe()
        self.set_value(ATTR_LAST_UPDATE, SEP.join((UPDATE_VALUE, group, str(row), str(column), str(value))))

    @expose_method({"group": "Path to existing group", "row": "Row index"})
    def remove_row(self, group, row):
        """
        Insert new row into group specified by group path
        @param group: group path
        @type group: string
        @param row: row index
        @type row: int
        """
        df = pd.read_hdf(self.filepath, key=group)
        df.drop(labels=row, axis=0, inplace=True)
        df.reset_index(drop=True, inplace=True)
        df.to_hdf(self.filepath, group, format="table", complevel=1)
        self.store_dataframe()
        self.set_value(ATTR_LAST_UPDATE, SEP.join((UPDATE_REMOVE_ROW, group, str(row))))

    def store_dataframe(self):
        with open(self.filepath, "rb") as fp:
            self.set_raw_value(ATTR_CONTENT, fp.read())



import time
from threading import Thread
from time import sleep

from kamzik3 import DeviceError, units
from kamzik3.constants import *
from kamzik3.devices.device import Device
from kamzik3.devices.observer import Observer
from kamzik3.macro.step import StepDeviceAttributeNumerical, StepParallel
from kamzik3.snippets.snippetsDecorators import expose_method


class DeviceVirtual(Device, Observer):

    configured = False

    def __init__(self, device_list, attribute_list, device_id=None, config=None):
        assert isinstance(device_list, list)
        assert isinstance(attribute_list, list)
        self.device_list = device_list
        self.attribute_list = attribute_list
        self.executed_step = None
        self.attached_callbacks = []
        super().__init__(device_id, config)
        for device in self.device_list:
            device.attach_attribute_callback(ATTR_STATUS, self.devices_status_update, key_filter=VALUE)

    def devices_status_update(self, value):
        if self.configured:
            return
        for device in self.device_list:
            if not device.in_statuses(READY_DEVICE_STATUSES):
                return
        self.connect()

    def connect(self, *args):
        try:
            self.connecting = True
            self.connected = False
            self.device_connection_poller.add_connecting_device(self)
            for device in self.device_list:
                if not device.in_statuses(READY_DEVICE_STATUSES):
                    self.connection_error = True
                    return False
            self.handle_connect_event()
        except DeviceError:
            self.logger.exception(u"Connection exception")
            return

    def handle_configuration(self):
        start_at = time.time()
        self.configured = True
        for device, attribute, callback in self.attached_callbacks:
            device.detach_attribute_callback(ATTR_STATUS, self.devices_status_update)

        for device in self.device_list:
            device.attach_observer(self)

        for attribute in self.attribute_list:
            device_attribute = self.device_list[0].get_attribute(attribute).attribute_copy()
            device_attribute.set_value_when_set_function = True
            device_attribute.set_function = lambda value, attr=attribute: self._set_attribute_value(attr, value)
            self.add_attribute(attribute, device_attribute)

            def callback(value, attr=attribute):
                self.attribute_changed(attr, value)
            self.device_list[0].attach_attribute_callback(attribute, callback, key_filter=VALUE)
            self.attached_callbacks.append((self.device_list[0], attribute, callback))

        self._config_commands()
        self._config_attributes()
        self.start_polling()
        self.set_status(STATUS_CONFIGURED)
        self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))

    def _set_attribute_value(self, attribute, value, callback=None):
        try:
            self.set_absolute_value(attribute, value)
        except DeviceError as e:
            self.logger.exception(e)
            raise

    def set_absolute_value(self, attribute, value):
        value_delta = value - self.device_list[0].get_value(attribute)
        self.set_relative_value(attribute, units.Quantity(value_delta, self.get_attribute(attribute)[UNIT]))

    def set_relative_value(self, attribute, value):
        step_list = []
        for index, device in enumerate(self.device_list):
            device_attribute = device.get_attribute(attribute)
            device_absolute_value = device_attribute.value() + value
            if not device_attribute.within_limits(device_absolute_value):
                raise DeviceError(
                    "New position error. New value of {} {} is not within limits.".format(device.device_id, attribute))
            step_list.append(StepDeviceAttributeNumerical(index, device.device_id, attribute, device_absolute_value,
                                                          positive_tolerance=device_attribute.positive_tolerance(),
                                                          negative_tolerance=device_attribute.negative_tolerance()))

        parallel_step = StepParallel("Set_Attribute_Step", step_list)
        Thread(target=self.execute_step, args=[parallel_step]).start()

    def execute_step(self, step):
        if self.executed_step is not None:
            step.error("Device is not ready")
        else:
            self.executed_step = step
            self.set_status(STATUS_BUSY)
            step.start()
            self.set_status(STATUS_IDLE)
        if step.get_state() == STATUS_ERROR:
            self.handle_command_error(u"Set attribute value", step.error_message)

        self.executed_step = None

    def subject_update(self, key, value, subject):
        # Don't update status when executing attribute change
        if self.executed_step is not None:
            return

        status_priorities = []
        for device in self.device_list:
            priority = STATUSES_PRIORITY.index(device[ATTR_STATUS][VALUE])
            status_priorities.append(priority)

        group_status = STATUSES_PRIORITY[min(status_priorities)]
        self.set_status(group_status)

    def attribute_changed(self, attribute, value):
        self.set_raw_value(attribute, value)

    @expose_method()
    def stop(self):
        if self.executed_step is not None:
            self.executed_step.stop()

    def disconnect(self):
        for device in self.device_list:
            device.detach_observer(self)
        for device, attribute, callback in self.attached_callbacks:
            device.detach_attribute_callback(attribute, callback)

        return super().disconnect()

    def close(self):
        self.set_status(STATUS_DISCONNECTED)
        if self.response_error or self.connection_error:
            self.connecting = False
            self.connected = False
            # self.reconnect()



import logging
from threading import Event, Thread

import kamzik3
from kamzik3 import DeviceError, units, MacroException
from kamzik3.constants import *
from kamzik3.devices.attribute import Attribute
from kamzik3.macro.common import Common


# TODO: Implement ramping of value step

class Step(Common):

    def __init__(self, common_id, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True,
                 on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
        self.steps_count = 1
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step".format(common_id))
        super(Step, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
                                   scanner=scanner, scanner_attributes=scanner_attributes)

    def start(self):
        super(Step, self).start()
        self.body()

        # Raise exception when step is in error state
        if self.get_state() == WARNING:
            if self.on_warning == ABORT or self.retry_count > 0:
                raise MacroException(self.warning_message)
            else:
                # Skipping here
                self.done()
                return

        # Raise exception when step is in error state
        if self.get_state() == ERROR:
            raise MacroException(self.error_message)
        if self.get_state() == STOPPED:
            return
        self.start_timer()
        if self.get_state() == STOPPED:
            return
        self.done()

    def repeat(self):
        super(Step, self).repeat()
        self.body()
        # Raise exception when step is in error state
        if self.get_state() == WARNING:
            if self.on_warning == ABORT or self.retry_count > 0:
                raise MacroException(self.warning_message)
            else:
                return
        # Raise exception when step is in error state
        if self.get_state() == ERROR:
            raise MacroException(self.error_message)
        if self.get_state() == STOPPED:
            return
        self.start_timer()
        if self.get_state() == STOPPED:
            return
        self.done()

    def body(self):
        raise NotImplementedError()

    def get_total_steps_count(self):
        return self.steps_count

    def get_total_points_count(self):
        return self.points_count

    def get_output_header(self):
        lines = [
            "Wait: {:~}".format(self.wait_after),
            "Repeat count: {}".format(self.repeat_count),
            "Retry: {}".format(self.retry_count),
            "Trigger log: {}".format(self.trigger_log),
        ]
        return lines


class StepSetpoint(Step):

    def __init__(self, common_id, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
                 scanner_attributes=None):
        self.setpoint = setpoint
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step".format(common_id))
        super(StepSetpoint, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning,
                                           timeout, scanner=scanner, scanner_attributes=scanner_attributes)

    def body(self):
        raise NotImplementedError()

    def check_value_on_setpoint(self, value):
        if value is None:
            return False
        return self.setpoint == value

    def get_output_header(self):
        lines = [
            "Setpoint: {}".format(self.setpoint),
        ]
        return lines + Step.get_output_header(self)


class StepDeviceAttribute(StepSetpoint):
    check_max_timeout = 2000
    check_timeout = 0
    device = None
    device_attribute = None

    def __init__(self, common_id, device_id, attribute, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"),
                 retry_count=0, trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
                 scanner_attributes=None):
        self.device_id = device_id
        self.attribute = Attribute.list_attribute(attribute)
        self.device_status_idle_flag = False
        self.device_setpoint_reached_flag = False
        self.check_event = Event()
        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
        super(StepDeviceAttribute, self).__init__(common_id, setpoint, repeat_count, wait_after, retry_count,
                                                  trigger_log, on_warning, timeout, scanner=scanner,
                                                  scanner_attributes=scanner_attributes)

    def body(self):
        self.check_timeout = 0
        try:
            self.logger.debug("Setting setpoint to {}".format(self.setpoint))
            self.device = kamzik3.session.get_device(self.device_id)
            self.device_attribute = self.device.get_attribute(self.attribute)

            if not self.device.in_statuses(READY_DEVICE_STATUSES):
                raise DeviceError(u"Device {} is not ready".format(self.device_id))
            # Attach callback for attribute and status changes
            self.device.attach_attribute_callback(self.attribute, self.check_attribute_set, key_filter=VALUE)
            self.device.attach_attribute_callback(ATTR_STATUS, self.check_device_status, key_filter=VALUE)
            # Set device's attribute to desired setpoint
            self.device.set_attribute(self.attribute + [VALUE], self.setpoint, callback=self.on_value_set)
            # Check if reached setpoint or timed out
            self.device_status_idle_flag = self.device.in_statuses(IDLE_DEVICE_STATUSES)
            self.device_setpoint_reached_flag = False

            while self.get_state() == BUSY and self.check_timeout <= self.check_max_timeout:
                self.check_event.wait(0.01)
                # To successfully finish step device has to be IDLE and attributes value within setpoints tolerance
                if self.device_status_idle_flag and self.device_setpoint_reached_flag:
                    # Set check event and break while loop
                    self.check_event.set()
                    break
                else:
                    self.check_timeout += 10
        except (DeviceError, AttributeError) as e:
            self.error(e)
        # Detach attribute and status callback
        self.device.detach_attribute_callback(self.attribute, self.check_attribute_set)
        self.device.detach_attribute_callback(ATTR_STATUS, self.check_device_status)

        # Raise exception if setpoint was not reached
        if self.get_state() != STOPPED and not self.check_event.isSet():
            description = "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id,
                                                                                        self.attribute, self.setpoint)
            self.warning(description)

        # Remove device and attributes
        self.device = None
        self.device_attribute = None

    def get_output(self):
        return self.device.get_attribute(self.attribute).value()

    def check_attribute_set(self, value):
        # Check if value is within setpoint tolerants
        if self.check_value_on_setpoint(self.device_attribute.value()):
            self.device_setpoint_reached_flag = True
        else:
            self.device_setpoint_reached_flag = False
        # Reset check_timeout
        if not self.device_status_idle_flag:
            self.check_timeout = 0

    def check_device_status(self, value):
        # Check if device is idle
        if value in IDLE_DEVICE_STATUSES:
            self.device_status_idle_flag = True
        else:
            self.device_status_idle_flag = False
        # Reset check_timeout
        self.check_timeout = 0

    def on_value_set(self, key, value):
        # Check for error when setting setpoint
        if key == RESPONSE_ERROR:
            self.error(value)

    def reset(self):
        self.device_status_idle_flag = False
        self.device_setpoint_reached_flag = False
        self.check_event.clear()
        self.retry_count = self.max_retry_count
        super(StepDeviceAttribute, self).reset()

    def get_output_header(self):
        lines = [
            "Device: {}".format(self.device_id),
            "Attribute: {}".format(self.attribute),
        ]

        return lines + StepSetpoint.get_output_header(self)

    def stop(self):
        if self.device is not None and self.get_state() == BUSY:
            self.device.stop()
        StepSetpoint.stop(self)

    def warning(self, warning_message):
        if self.device is not None and self.get_state() == BUSY:
            self.device.stop()
        StepSetpoint.warning(self)


class StepSetpointNumerical(Step):

    def __init__(self, common_id, setpoint, negative_tolerance=None, positive_tolerance=None, repeat_count=0,
                 wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
                 timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
        assert isinstance(setpoint, units.Quantity)

        self.setpoint = setpoint
        self.negative_tolerance = negative_tolerance
        if self.negative_tolerance is None:
            self.negative_tolerance = units.Quantity(0, self.setpoint.u)
        self.positive_tolerance = positive_tolerance
        if self.positive_tolerance is None:
            self.positive_tolerance = units.Quantity(0, self.setpoint.u)
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step".format(common_id))
        super(StepSetpointNumerical, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log,
                                                    on_warning, timeout, scanner=scanner,
                                                    scanner_attributes=scanner_attributes)

    def body(self):
        raise NotImplementedError()

    def get_output(self):
        return self.device.get_attribute(self.attribute).value()

    def check_value_on_setpoint(self, value):
        # Check value is within +/- setpoints tolerance
        assert isinstance(value, units.Quantity)

        if value is None:
            return False
        return (self.setpoint - self.negative_tolerance) <= value <= (self.setpoint + self.positive_tolerance)

    def get_output_header(self):
        lines = [
            "Setpoint: {:~}".format(self.setpoint),
            "Negative tolerance: {:~}".format(self.negative_tolerance),
            "Positive tolerance: {:~}".format(self.positive_tolerance),
        ]
        return lines + Step.get_output_header(self)


class StepDeviceAttributeNumerical(StepSetpointNumerical):
    check_max_timeout = 2000
    check_timeout = 0
    device = None
    device_attribute = None

    def __init__(self, common_id, device_id, attribute, setpoint, negative_tolerance=None, positive_tolerance=None,
                 repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
                 timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
        self.device_id = device_id
        self.attribute = Attribute.list_attribute(attribute)
        self.device_status_idle_flag = False
        self.device_setpoint_reached_flag = False
        self.check_event = Event()
        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
        super(StepDeviceAttributeNumerical, self).__init__(common_id, setpoint, negative_tolerance, positive_tolerance,
                                                           repeat_count, wait_after, retry_count, trigger_log,
                                                           on_warning, timeout, scanner=scanner,
                                                           scanner_attributes=scanner_attributes)

    def body(self):
        self.check_timeout = 0
        try:
            self.logger.debug("Setting setpoint to {} -{} / +{}".format(self.setpoint, self.negative_tolerance,
                                                                        self.positive_tolerance))
            self.device = kamzik3.session.get_device(self.device_id)
            self.device_attribute = self.device.get_attribute(self.attribute)

            if not self.device.in_statuses(READY_DEVICE_STATUSES):
                raise DeviceError(u"Device {} is not ready".format(self.device_id))
            if self.check_value_on_setpoint(self.device_attribute.value()):
                return
            # Attach callback for attribute and status changes
            self.device.attach_attribute_callback(self.attribute, self.check_attribute_set, key_filter=VALUE)
            self.device.attach_attribute_callback(ATTR_STATUS, self.check_device_status, key_filter=VALUE)
            # Set device's attribute to desired setpoint
            setpoint = self.device_attribute.convert_units(self.setpoint)
            self.device.set_attribute(self.attribute + [VALUE], setpoint.m, callback=self.on_value_set)
            # Check if reached setpoint or timed out
            self.device_status_idle_flag = self.device.in_statuses(IDLE_DEVICE_STATUSES)
            self.device_setpoint_reached_flag = False

            if self.check_value_on_setpoint(self.device_attribute.value()):
                self.check_event.set()
            else:
                while self.get_state() == BUSY and self.check_timeout <= self.check_max_timeout:
                    self.check_event.wait(0.01)
                    # To successfully finish step device has to be IDLE and attributes value within setpoints tolerance
                    if self.device_status_idle_flag and self.device_setpoint_reached_flag:
                        # Set check event and break while loop
                        self.check_event.set()
                        break
                    else:
                        # Reset check_timeout
                        self.check_timeout += 10
        except (DeviceError, AttributeError) as e:
            self.error(e)
        # Raise warning if setpoint was not reached
        if self.get_state() != STOPPED and not self.check_event.isSet():
            description = "{}'s attribute setpoint {}: {} could not be reached.".format(self.device.device_id,
                                                                                        self.attribute, self.setpoint)
            self.warning(description)

        # Detach attribute and status callback
        self.device.detach_attribute_callback(self.attribute, self.check_attribute_set)
        self.device.detach_attribute_callback(ATTR_STATUS, self.check_device_status)
        # Remove device and attributes
        self.device = None
        self.device_attribute = None

    def get_output(self):
        device = kamzik3.session.get_device(self.device_id)
        return "{}".format(device.get_attribute(self.attribute).value())

    def check_attribute_set(self, value):
        # Check if value is within setpoint tolerants
        if self.check_value_on_setpoint(self.device_attribute.value()):
            self.device_setpoint_reached_flag = True
        else:
            self.device_setpoint_reached_flag = False
        # Reset check_timeout when device is busy
        if not self.device_status_idle_flag:
            self.check_timeout = 0

    def check_device_status(self, value):
        # Check if device is idle
        if value in IDLE_DEVICE_STATUSES:
            self.device_status_idle_flag = True
        else:
            self.device_status_idle_flag = False

    def on_value_set(self, key, value):
        # Check for error when setting setpoint
        if key == RESPONSE_ERROR:
            self.error(value)

    def reset(self):
        self.device_status_idle_flag = False
        self.device_setpoint_reached_flag = False
        self.check_event.clear()
        super(StepDeviceAttributeNumerical, self).reset()

    def get_output_header(self):
        lines = [
            "Type: {}".format("Attribute"),
            "Device: {}".format(self.device_id),
            "Attribute: {}".format(self.attribute),
        ]

        return lines + StepSetpointNumerical.get_output_header(self)

    def stop(self):
        device = kamzik3.session.get_device(self.device_id)
        device.stop()
        device.wait_for_status(STATUS_IDLE)
        StepSetpointNumerical.stop(self)

    def warning(self, warning_message):
        device = kamzik3.session.get_device(self.device_id)
        device.stop()
        device.wait_for_status(STATUS_IDLE)
        StepSetpointNumerical.warning(self, warning_message)


class StepDeviceMethod(Step):
    check_max_timeout = 2000
    check_timeout = 0
    device = None
    device_method = None
    retry_timeout = 1000

    def __init__(self, common_id, device_id, method, method_parameters, repeat_count=0,
                 wait_after=units.Quantity(0, "s"), retry_count=0, busy_timeout=units.Quantity(1, "s"),
                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
                 scanner_attributes=None):
        self.device_id = device_id
        self.method = method
        self.method_parameters = method_parameters
        self.device_status_idle_flag = False
        self.check_event = Event()
        # self.timeout_lock = Lock()  # Prevents race on setting check_timeout
        self.busy_timeout = busy_timeout
        if not hasattr(self, "logger"):
            self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, method))
        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
                      scanner=scanner, scanner_attributes=scanner_attributes)

    def body(self):
        self.check_timeout = 0
        try:
            self.logger.debug("Executing method {}".format(self.method))
            self.device = kamzik3.session.get_device(self.device_id)
            self.device_method = getattr(self.device, self.method)
            if not self.device.in_statuses(READY_DEVICE_STATUSES):
                raise DeviceError(u"Device {} is not ready".format(self.device_id))
            # Attach callback for attribute and status changes
            self.device.attach_attribute_callback(ATTR_STATUS, self.check_device_status, key_filter=VALUE)
            # Set device's attribute to desired setpoint
            self.device_method(**self.method_parameters)
            # Check if reached setpoint or timed out
            self.device_status_idle_flag = self.device.in_statuses(IDLE_DEVICE_STATUSES)
            wait_for_status_busy_timeout = self.busy_timeout.to("s").m / 0.1
            while self.get_state() == BUSY and self.device_status_idle_flag and wait_for_status_busy_timeout > 0:
                self.check_event.wait(0.1)
                wait_for_status_busy_timeout -= 1
            while self.get_state() == BUSY and not self.device_status_idle_flag:
                self.check_event.wait(0.1)
            self.check_event.set()
        except (DeviceError, AttributeError) as e:
            self.error(e)

        # Detach attribute and status callback
        self.device.detach_attribute_callback(ATTR_STATUS, self.check_device_status)
        # Remove device and attributes
        self.device = None
        self.device_method = None

    def check_device_status(self, value):
        # Check if device is idle
        if value in IDLE_DEVICE_STATUSES:
            self.device_status_idle_flag = True
        else:
            self.device_status_idle_flag = False

    def retry(self, retry_reason):
        try:
            # Make sure we tried to stop whatever is going on before retrying step again.
            kamzik3.session.get_device(self.device_id).stop()
        except DeviceError:
            self.logger.error("Device could not be stopped")
        Step.retry(self, retry_reason)

    def on_value_set(self, key, value):
        # Check for error when setting setpoint
        if key == RESPONSE_ERROR:
            self.error(value)

    def reset(self):
        self.device_status_idle_flag = False
        self.check_event.clear()
        Step.reset(self)

    def warning(self, warning_message):
        device = kamzik3.session.get_device(self.device_id)
        device.stop()
        device.wait_for_status(STATUS_IDLE)
        Step.warning(self, "{} executing {} on {}".format(warning_message, self.method, self.device_id))

    def stop(self):
        device = kamzik3.session.get_device(self.device_id)
        device.stop()
        device.wait_for_status(STATUS_IDLE)
        Step.stop(self)

    def get_output_header(self):
        lines = [
            "Type: {}".format("Method"),
            "Device: {}".format(self.device_id),
            "Busy timeout: {}".format(self.busy_timeout),
            "Method: {}".format(self.method),
        ]
        for attribute, value in self.method_parameters.items():
            lines.append("{}: {}".format(attribute, value))

        return lines + Step.get_output_header(self)


# todo: Revise Parallel step
class StepParallel(Step):

    def __init__(self, common_id, step_list, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
                 scanner_attributes=None):
        assert isinstance(step_list, list)
        self.step_list = step_list
        for step in self.step_list:
            if not isinstance(step, Step):
                raise TypeError("Step {} is not type of Step".format(step))
        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
                      scanner=scanner, scanner_attributes=scanner_attributes)

    def body(self):
        step_threads = []
        # Start each step in separate thread
        for step in self.step_list:
            step_thread = Thread(target=step.start)
            step_threads.append(step_thread)
            step_thread.start()

        # Check if parallel step was not aborted
        while len(step_threads) != 0 and self.get_state() == BUSY:
            for step_thread in step_threads[:]:
                if step_thread.is_alive():
                    # Try to join thread with 100 ms
                    step_thread.join(timeout=0.1)
                else:
                    step_threads.remove(step_thread)

            for step in self.step_list:
                if step.get_state() == ERROR:
                    self.set_state(ERROR)
                    self.error_message = step.error_message
                    raise MacroException(self.error_message)
                elif step.get_state() == STOPPED:
                    self.set_state(STOPPED)

        if self.get_state() == STOPPED:
            for step in self.step_list:
                if step.get_state() == BUSY:
                    step.stop()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'logViewTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(911, 604)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/kamzik.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Form.setWindowIcon(icon)
        Form.setStyleSheet("QGroupBox {\n"
"    border: 1px solid gray;\n"
"    margin-top: 0.5em;\n"
" font-weight:bold\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"    left: 10px;\n"
"    padding: 0 3px 0 3px;\n"
" font-weight:bold\n"
"   \n"
"}\n"
"\n"
"\n"
"QSplitter::handle:horizontal {\n"
"  image: url(:/icons/icons/splitter.png);\n"
"  width:13px;\n"
"  height:13px;\n"
"  background: #f6b442;\n"
"}\n"
"\n"
"")
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setHandleWidth(11)
        self.splitter.setObjectName("splitter")
        self.tabs_tools = QtWidgets.QTabWidget(self.splitter)
        self.tabs_tools.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabs_tools.setObjectName("tabs_tools")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.pushButton = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton.sizePolicy().hasHeightForWidth())
        self.pushButton.setSizePolicy(sizePolicy)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_6.addWidget(self.pushButton, 0, 2, 1, 1)
        self.input_max_selected_rows = QtWidgets.QSpinBox(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.input_max_selected_rows.sizePolicy().hasHeightForWidth())
        self.input_max_selected_rows.setSizePolicy(sizePolicy)
        self.input_max_selected_rows.setMinimum(1)
        self.input_max_selected_rows.setMaximum(25)
        self.input_max_selected_rows.setProperty("value", 25)
        self.input_max_selected_rows.setObjectName("input_max_selected_rows")
        self.gridLayout_6.addWidget(self.input_max_selected_rows, 0, 1, 1, 1)
        self.input_header_items = QtWidgets.QListView(self.groupBox_2)
        self.input_header_items.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.input_header_items.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.input_header_items.setObjectName("input_header_items")
        self.gridLayout_6.addWidget(self.input_header_items, 2, 0, 1, 3)
        self.label = QtWidgets.QLabel(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout_6.addWidget(self.label, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox_2, 1, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_3.setContentsMargins(9, 9, 9, 9)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.button_list_view = QtWidgets.QPushButton(self.groupBox_3)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/viewList.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_list_view.setIcon(icon1)
        self.button_list_view.setCheckable(True)
        self.button_list_view.setChecked(True)
        self.button_list_view.setAutoRepeat(False)
        self.button_list_view.setAutoExclusive(True)
        self.button_list_view.setDefault(True)
        self.button_list_view.setObjectName("button_list_view")
        self.horizontalLayout_2.addWidget(self.button_list_view)
        self.button_zoom_view = QtWidgets.QPushButton(self.groupBox_3)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/viewZoom.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_zoom_view.setIcon(icon2)
        self.button_zoom_view.setCheckable(True)
        self.button_zoom_view.setAutoExclusive(True)
        self.button_zoom_view.setObjectName("button_zoom_view")
        self.horizontalLayout_2.addWidget(self.button_zoom_view)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 0, 0, 1, 2)
        self.tabs_view_tools = QtWidgets.QStackedWidget(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabs_view_tools.sizePolicy().hasHeightForWidth())
        self.tabs_view_tools.setSizePolicy(sizePolicy)
        self.tabs_view_tools.setObjectName("tabs_view_tools")
        self.page_zoom = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page_zoom.sizePolicy().hasHeightForWidth())
        self.page_zoom.setSizePolicy(sizePolicy)
        self.page_zoom.setObjectName("page_zoom")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.page_zoom)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_6 = QtWidgets.QLabel(self.page_zoom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_7.addWidget(self.label_6, 2, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.page_zoom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_16.sizePolicy().hasHeightForWidth())
        self.label_16.setSizePolicy(sizePolicy)
        self.label_16.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label_16.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout_7.addWidget(self.label_16, 3, 0, 1, 1)
        self.input_to_dt = QtWidgets.QDateTimeEdit(self.page_zoom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.input_to_dt.sizePolicy().hasHeightForWidth())
        self.input_to_dt.setSizePolicy(sizePolicy)
        self.input_to_dt.setReadOnly(True)
        self.input_to_dt.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        self.input_to_dt.setObjectName("input_to_dt")
        self.gridLayout_7.addWidget(self.input_to_dt, 2, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.page_zoom)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_7.addWidget(self.label_2, 0, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.page_zoom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.gridLayout_7.addWidget(self.label_5, 1, 0, 1, 1)
        self.input_from_dt = QtWidgets.QDateTimeEdit(self.page_zoom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.input_from_dt.sizePolicy().hasHeightForWidth())
        self.input_from_dt.setSizePolicy(sizePolicy)
        self.input_from_dt.setReadOnly(True)
        self.input_from_dt.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        self.input_from_dt.setObjectName("input_from_dt")
        self.gridLayout_7.addWidget(self.input_from_dt, 1, 1, 1, 1)
        self.readout_diff_dt = QtWidgets.QLineEdit(self.page_zoom)
        self.readout_diff_dt.setReadOnly(True)
        self.readout_diff_dt.setObjectName("readout_diff_dt")
        self.gridLayout_7.addWidget(self.readout_diff_dt, 3, 1, 1, 1)
        self.input_master_row = QtWidgets.QComboBox(self.page_zoom)
        self.input_master_row.setObjectName("input_master_row")
        self.gridLayout_7.addWidget(self.input_master_row, 0, 1, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.pushButton_2 = QtWidgets.QPushButton(self.page_zoom)
        self.pushButton_2.setObjectName("pushButton_2")
        self.horizontalLayout_4.addWidget(self.pushButton_2)
        self.gridLayout_7.addLayout(self.horizontalLayout_4, 5, 0, 1, 2)
        self.input_zoom_follow = QtWidgets.QCheckBox(self.page_zoom)
        self.input_zoom_follow.setChecked(True)
        self.input_zoom_follow.setObjectName("input_zoom_follow")
        self.gridLayout_7.addWidget(self.input_zoom_follow, 4, 0, 1, 2)
        self.tabs_view_tools.addWidget(self.page_zoom)
        self.page_list = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page_list.sizePolicy().hasHeightForWidth())
        self.page_list.setSizePolicy(sizePolicy)
        self.page_list.setObjectName("page_list")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.page_list)
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.input_view_scale_plots = QtWidgets.QCheckBox(self.page_list)
        self.input_view_scale_plots.setChecked(True)
        self.input_view_scale_plots.setObjectName("input_view_scale_plots")
        self.gridLayout_8.addWidget(self.input_view_scale_plots, 0, 0, 1, 1)
        self.input_view_plot_height = QtWidgets.QSpinBox(self.page_list)
        self.input_view_plot_height.setEnabled(False)
        self.input_view_plot_height.setMinimum(5)
        self.input_view_plot_height.setMaximum(100)
        self.input_view_plot_height.setProperty("value", 25)
        self.input_view_plot_height.setObjectName("input_view_plot_height")
        self.gridLayout_8.addWidget(self.input_view_plot_height, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_8.addItem(spacerItem2, 2, 0, 1, 1)
        self.tabs_view_tools.addWidget(self.page_list)
        self.gridLayout_3.addWidget(self.tabs_view_tools, 2, 0, 1, 1)
        self.line = QtWidgets.QFrame(self.groupBox_3)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_3.addWidget(self.line, 1, 0, 1, 2)
        self.gridLayout_2.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.tabs_tools.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.label_3 = QtWidgets.QLabel(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.gridLayout_9.addWidget(self.label_3, 2, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.gridLayout_9.addWidget(self.label_4, 3, 0, 1, 1)
        self.input_plot_height = QtWidgets.QSpinBox(self.groupBox_4)
        self.input_plot_height.setMaximum(99999999)
        self.input_plot_height.setProperty("value", 600)
        self.input_plot_height.setObjectName("input_plot_height")
        self.gridLayout_9.addWidget(self.input_plot_height, 3, 1, 1, 1)
        self.input_plot_width = QtWidgets.QSpinBox(self.groupBox_4)
        self.input_plot_width.setMaximum(99999999)
        self.input_plot_width.setProperty("value", 1600)
        self.input_plot_width.setObjectName("input_plot_width")
        self.gridLayout_9.addWidget(self.input_plot_width, 2, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.pushButton_4 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_4.setObjectName("pushButton_4")
        self.horizontalLayout.addWidget(self.pushButton_4)
        self.pushButton_3 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_3.setObjectName("pushButton_3")
        self.horizontalLayout.addWidget(self.pushButton_3)
        self.gridLayout_9.addLayout(self.horizontalLayout, 4, 1, 1, 1)
        self.gridLayout_10.addWidget(self.groupBox_4, 0, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_10.addItem(spacerItem4, 1, 0, 1, 1)
        self.tabs_tools.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.groupBox = QtWidgets.QGroupBox(self.tab_3)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.input_windowed_plots = QtWidgets.QCheckBox(self.groupBox)
        self.input_windowed_plots.setChecked(True)
        self.input_windowed_plots.setObjectName("input_windowed_plots")
        self.gridLayout_5.addWidget(self.input_windowed_plots, 0, 0, 1, 1)
        self.input_keep_tiled = QtWidgets.QCheckBox(self.groupBox)
        self.input_keep_tiled.setChecked(True)
        self.input_keep_tiled.setObjectName("input_keep_tiled")
        self.gridLayout_5.addWidget(self.input_keep_tiled, 1, 0, 1, 1)
        self.gridLayout_4.addWidget(self.groupBox, 0, 0, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem5, 1, 0, 1, 1)
        self.tabs_tools.addTab(self.tab_3, "")
        self.mdi_area = QtWidgets.QMdiArea(self.splitter)
        self.mdi_area.setLineWidth(0)
        self.mdi_area.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.mdi_area.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.mdi_area.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.mdi_area.setViewMode(QtWidgets.QMdiArea.SubWindowView)
        self.mdi_area.setDocumentMode(False)
        self.mdi_area.setTabsClosable(False)
        self.mdi_area.setObjectName("mdi_area")
        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.tabs_tools.setCurrentIndex(0)
        self.tabs_view_tools.setCurrentIndex(1)
        self.input_max_selected_rows.valueChanged['int'].connect(Form.slot_set_max_selected_rows)
        self.pushButton.clicked.connect(self.input_header_items.clearSelection)
        self.input_windowed_plots.toggled['bool'].connect(Form.slot_set_windowed_plots)
        self.input_header_items.pressed['QModelIndex'].connect(Form.slot_selection_changed)
        self.input_master_row.currentTextChanged['QString'].connect(Form.slot_master_row_changed)
        self.splitter.splitterMoved['int','int'].connect(Form.slot_splitter_moved)
        self.input_view_scale_plots.toggled['bool'].connect(self.input_view_plot_height.setDisabled)
        self.button_list_view.toggled['bool'].connect(Form.slot_set_list_mode)
        self.button_zoom_view.toggled['bool'].connect(Form.slot_set_zoom_mode)
        self.input_view_plot_height.editingFinished.connect(Form.slot_set_list_min_height)
        self.input_view_scale_plots.clicked.connect(Form.slot_set_list_min_height)
        self.pushButton.clicked.connect(Form.slot_selection_changed)
        self.pushButton_2.clicked.connect(Form.slot_reset_zoom)
        self.input_keep_tiled.toggled['bool'].connect(Form.slot_keep_tiled_changed)
        self.pushButton_3.clicked.connect(Form.slot_export_to_pdf)
        self.pushButton_4.clicked.connect(Form.slot_export_to_csv)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.groupBox_2.setTitle(_translate("Form", "Rows"))
        self.pushButton.setText(_translate("Form", "Clear"))
        self.label.setText(_translate("Form", "Max selected rows:"))
        self.groupBox_3.setTitle(_translate("Form", "View type"))
        self.button_list_view.setText(_translate("Form", " List"))
        self.button_zoom_view.setText(_translate("Form", " Zoom"))
        self.label_6.setText(_translate("Form", "To datetime:"))
        self.label_16.setText(_translate("Form", "Diff:"))
        self.input_to_dt.setDisplayFormat(_translate("Form", "MM/dd/yy hh:mm:ss"))
        self.label_2.setText(_translate("Form", "Master row:"))
        self.label_5.setText(_translate("Form", "From datetime:"))
        self.input_from_dt.setDisplayFormat(_translate("Form", "MM/dd/yy hh:mm:ss"))
        self.pushButton_2.setText(_translate("Form", "Reset zoom"))
        self.input_zoom_follow.setText(_translate("Form", "Zoom follow active window"))
        self.input_view_scale_plots.setText(_translate("Form", "Scale plots to fit windows"))
        self.input_view_plot_height.setSuffix(_translate("Form", " %"))
        self.input_view_plot_height.setPrefix(_translate("Form", "Plot height: "))
        self.tabs_tools.setTabText(self.tabs_tools.indexOf(self.tab), _translate("Form", "View"))
        self.groupBox_4.setTitle(_translate("Form", "Plots export"))
        self.label_3.setText(_translate("Form", "Width:"))
        self.label_4.setText(_translate("Form", "Height:"))
        self.input_plot_height.setSuffix(_translate("Form", " px"))
        self.input_plot_width.setSuffix(_translate("Form", " px"))
        self.pushButton_4.setText(_translate("Form", "Export to csv"))
        self.pushButton_3.setText(_translate("Form", "Export to pdf"))
        self.tabs_tools.setTabText(self.tabs_tools.indexOf(self.tab_2), _translate("Form", "Export"))
        self.groupBox.setTitle(_translate("Form", "Subplots"))
        self.input_windowed_plots.setText(_translate("Form", "Windowed plots"))
        self.input_keep_tiled.setText(_translate("Form", "Keep tiled"))
        self.tabs_tools.setTabText(self.tabs_tools.indexOf(self.tab_3), _translate("Form", "Settings"))
from kamzik3 import resource_kamzik3_rc



from kamzik3.devices.observer import Observer


class Subject(object):
    """
    Implements Subject which server Observer updates and notifications.
    """

    _observers = None

    def __init__(self):
        if not self._observers:
            self._observers = []

    def notify(self, key, value):
        """
        Notify all observers with an subject_update
        :param key: mixed
        :param value: mixed
        :return: None
        """
        # with self.notify_lock:
        for observer in self._observers[:]:
            observer.subject_update(key, value, self)

    def attach_observer(self, observer):
        """
        Attach new observer.
        :param observer: Observer
        :return: None
        """
        if observer not in self._observers:
            if isinstance(observer, Observer):
                self._observers.append(observer)
            else:
                raise AttributeError(u"observer has to implement kamzik3.device.observer.Observer interface")

        self.handle_observer_attached(observer)

    def detach_observer(self, observer):
        """
        Detach Observer from list of observers
        :param observer: Observer
        :return: None
        """
        if observer in self._observers:
            self._observers.remove(observer)
            self.handle_observer_detached(observer)

    def handle_observer_attached(self, observer):
        """
        Hook to be fired after attached Observer
        :param observer: Observer
        :return: None
        """
        raise NotImplementedError("Has to be implemented")

    def handle_observer_detached(self, observer):
        """
        Hook to be fired after detached Observer
        :param observer: Observer
        :return: None
        """
        raise NotImplementedError("Has to be implemented")



from PyQt5.QtCore import Qt, QPoint, QSize
from PyQt5.QtGui import QPainter, QColor, QMovie
from PyQt5.QtWidgets import QWidget

from kamzik3.gui.templates.loadingOverlayTemplate import Ui_Form


class LoadingOverlayWidget(QWidget, Ui_Form):
    overlayed_widget = None
    cancel_function = None
    canceled = False

    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.setupUi(self)

        movie = QMovie(":/icons/icons/loading.gif")
        movie.setScaledSize(QSize(128, 128))
        movie.start()
        self.image_loading.setMovie(movie)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.hide()

    def paintEvent(self, QPaintEvent):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(255, 255, 255, 200))
        painter.drawRect(0, 0, self.width(), self.height())

    def set_over(self, widget, message="Loading...", cancel_function=None):
        if cancel_function is not None:
            self.button_cancel.show()
            self.button_cancel.setEnabled(True)
        else:
            self.button_cancel.hide()
        self.canceled = False
        self.overlayed_widget = widget
        self.cancel_function = cancel_function
        self.label_loading.setText(message)
        self.show()
        self.resize(widget.rect().width() + 1, widget.rect().height() + 1)

    def cancel(self):
        self.button_cancel.setEnabled(False)
        if callable(self.cancel_function):
            self.cancel_function()
        self.canceled = True

    def hide(self):
        self.overlayed_widget = None
        QWidget.hide(self)

    def set_overlay_size(self):
        if self.overlayed_widget is not None:
            widget_pos = self.overlayed_widget.mapToGlobal(self.overlayed_widget.rect().topLeft())
            window_pos = self.window().mapToGlobal(self.window().rect().topLeft())
            rel_x = widget_pos.x() - window_pos.x()
            rel_y = widget_pos.y() - window_pos.y()
            self.setGeometry(self.overlayed_widget.geometry())
            self.move(QPoint(rel_x, rel_y))

    def resizeEvent(self, QResizeEvent):
        self.set_overlay_size()
        QWidget.resizeEvent(self, QResizeEvent)

    def slot_set_progress(self, value):
        self.label_progress.setText("{}%".format(round(value, 1)))



import numpy as np
from PyQt5.QtCore import pyqtSlot, Qt
from PyQt5.QtWidgets import QLabel, QWidget, QHBoxLayout

from kamzik3 import GuiTypeError
from kamzik3 import units
from kamzik3.constants import *
from kamzik3.gui.attributeWidgets.attributeBoolWidget import AttributeBoolWidget
from kamzik3.gui.attributeWidgets.attributeComboboxWidget import AttributeComboxWidget
from kamzik3.gui.attributeWidgets.attributeFloatWidget import AttributeFloatWidget
from kamzik3.gui.attributeWidgets.attributeImageWidget import AttributeImageWidget
from kamzik3.gui.attributeWidgets.attributeIntWidget import AttributeIntWidget
from kamzik3.gui.attributeWidgets.attributeStringWidget import AttributeStringWidget
from kamzik3.gui.attributeWidgets.attributeTextWidget import AttributeTextWidget
from kamzik3.gui.attributeWidgets.attributeUnsignedIntWidget import AttributeUnsignedIntWidget


class AttributeDisplayWidget(QWidget):
    attribute_widget = None
    label_widget = None
    input_widget = None
    unit_widget = None

    def __init__(self, title, attribute, config=None, parent=None):
        self.attribute = attribute
        self.config = config
        self.title = title
        QWidget.__init__(self, parent=parent)
        self.setupUi()

    def setupUi(self, parent=None):
        if self.attribute_widget is not None:
            self.attribute_widget.attribute = self.attribute
            return

        self.label_widget = QLabel(u"{}:".format(self.title))
        if self.attribute[TYPE] == TYPE_LIST:
            if TYPE_LIST_VALUES in self.attribute:
                self.attribute_widget = AttributeComboxWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.activated.connect(self.on_value_set)
            else:
                self.attribute_widget = AttributeStringWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.returnPressed.connect(
                    lambda attribute=self.attribute_widget.input_widget: self.on_value_set(list(attribute.text()))
                )
        elif self.attribute[TYPE] == TYPE_ARRAY:
            if self.attribute.get(DISPLAY) == IMAGE:
                self.attribute_widget = AttributeImageWidget(self.attribute, self.config)
            else:
                self.attribute_widget = AttributeTextWidget(self.attribute, self.config)
        else:
            np_type = np.dtype(self.attribute[TYPE])
            if np_type == np.str:
                self.attribute_widget = AttributeStringWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.returnPressed.connect(
                    lambda attribute=self.attribute_widget.input_widget: self.on_value_set(attribute.text())
                )
            elif np_type in (np.uint8, np.uint16, np.uint32, np.uint64):
                self.attribute_widget = AttributeUnsignedIntWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.returnPressed.connect(self.on_value_set)
            elif np_type in (np.int8, np.int16, np.int32, np.int64):
                self.attribute_widget = AttributeIntWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.returnPressed.connect(self.on_value_set)
            elif np_type in (np.float16, np.float32, np.float64):
                self.attribute_widget = AttributeFloatWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.returnPressed.connect(self.on_value_set)
            elif np_type == np.bool:
                self.attribute_widget = AttributeBoolWidget(self.attribute, self.config)
                self.attribute_widget.input_widget.clicked.connect(self.on_value_set)
            else:
                raise GuiTypeError(u"Unknown type of attribute")

        if self.config is not None and self.config.get("label_width"):
            self.label_widget.setMinimumWidth(self.config.get("label_width"))
            self.label_widget.setAlignment(Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter)

        self.input_widget = self.attribute_widget.input_widget
        self.unit_widget = self.attribute_widget.unit_widget

        if self.attribute.get(DESCRIPTION) is not None:
            self.label_widget.setToolTip(self.attribute[DESCRIPTION])

        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.label_widget)
        layout.addWidget(self.input_widget)
        layout.addWidget(self.unit_widget)

    def get_attribute_value(self):
        return self.attribute_widget.get_attribute_value()

    def get_widget_value(self):
        return self.attribute_widget.get_widget_value()

    def set_value(self, value):
        return self.attribute_widget.set_value(value)

    def get_minimum(self):
        return self.attribute_widget.get_minimum()

    def get_maximum(self):
        return self.attribute_widget.get_maximum()

    def set_minimum(self, value):
        return self.attribute_widget.set_minimum(value)

    def set_maximum(self, value):
        return self.attribute_widget.set_maximum(value)

    def on_value_set(self, value):
        self.attribute[VALUE] = self.attribute_widget.attribute_type_cast(value)

    def __add__(self, value):
        assert isinstance(value, units.Quantity)
        current_value = self.attribute.value().to(self.get_unit())
        new_value = current_value + value
        if new_value > self.get_maximum():
            new_value = self.get_maximum()
        elif new_value < self.get_minimum():
            new_value = self.get_minimum()
        # self.set_value(new_value)
        self.on_value_set(new_value)
        return self

    def __sub__(self, value):
        assert isinstance(value, units.Quantity)
        current_value = self.attribute.value().to(self.get_unit())
        new_value = current_value - value
        if new_value > self.get_maximum():
            new_value = self.get_maximum()
        elif new_value < self.get_minimum():
            new_value = self.get_minimum()
        # self.set_value(new_value)
        self.on_value_set(new_value)
        return self

    @pyqtSlot("QString")
    def set_unit(self, unit=None, reset=False):
        self.unit_widget.set_unit(unit)

    @pyqtSlot("QString")
    def get_unit(self):
        return self.attribute_widget.unit

    def close(self):
        if self.label_widget is not None:
            self.label_widget.setParent(None)
        if self.attribute_widget is not None:
            self.attribute_widget.close()
            self.attribute_widget.setParent(None)
        self.attribute_widget.deleteLater()
        self.label_widget.deleteLater()
        self.attribute = None
        self.config = None
        return QWidget(self)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'loadingOverlayTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.14.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(158, 214)
        Form.setWindowOpacity(1.0)
        Form.setAutoFillBackground(False)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 0, 1, 1, 1)
        self.button_cancel = QtWidgets.QPushButton(Form)
        self.button_cancel.setObjectName("button_cancel")
        self.gridLayout.addWidget(self.button_cancel, 3, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 4, 0, 1, 3)
        spacerItem2 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 2, 2, 1, 1)
        self.label_loading = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_loading.setFont(font)
        self.label_loading.setAlignment(QtCore.Qt.AlignCenter)
        self.label_loading.setObjectName("label_loading")
        self.gridLayout.addWidget(self.label_loading, 1, 1, 1, 1)
        self.frame = QtWidgets.QFrame(Form)
        self.frame.setMinimumSize(QtCore.QSize(128, 128))
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setLineWidth(0)
        self.frame.setObjectName("frame")
        self.image_loading = QtWidgets.QLabel(self.frame)
        self.image_loading.setGeometry(QtCore.QRect(0, 0, 128, 128))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.image_loading.sizePolicy().hasHeightForWidth())
        self.image_loading.setSizePolicy(sizePolicy)
        self.image_loading.setMinimumSize(QtCore.QSize(128, 128))
        self.image_loading.setMaximumSize(QtCore.QSize(128, 128))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.image_loading.setFont(font)
        self.image_loading.setLineWidth(0)
        self.image_loading.setText("")
        self.image_loading.setScaledContents(True)
        self.image_loading.setAlignment(QtCore.Qt.AlignCenter)
        self.image_loading.setObjectName("image_loading")
        self.label_progress = QtWidgets.QLabel(self.frame)
        self.label_progress.setGeometry(QtCore.QRect(0, 0, 128, 128))
        font = QtGui.QFont()
        font.setPointSize(15)
        self.label_progress.setFont(font)
        self.label_progress.setLineWidth(0)
        self.label_progress.setText("")
        self.label_progress.setAlignment(QtCore.Qt.AlignCenter)
        self.label_progress.setObjectName("label_progress")
        self.gridLayout.addWidget(self.frame, 2, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 2, 0, 1, 1)

        self.retranslateUi(Form)
        self.button_cancel.clicked.connect(Form.cancel)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.button_cancel.setText(_translate("Form", "Cancel"))
        self.label_loading.setText(_translate("Form", "Loading..."))
from kamzik3 import resource_kamzik3_rc